// This file was generated by gir (https://github.com/gtk-rs/gir)
// from
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use crate::ffi;
use glib::prelude::*;

glib::wrapper! {
    #[doc(alias = "SoupSessionFeature")]
    pub struct SessionFeature(Interface<ffi::SoupSessionFeature, ffi::SoupSessionFeatureInterface>);

    match fn {
        type_ => || ffi::soup_session_feature_get_type(),
    }
}

impl SessionFeature {
    pub const NONE: Option<&'static SessionFeature> = None;
}

pub trait SessionFeatureExt: IsA<SessionFeature> + 'static {}

impl<O: IsA<SessionFeature>> SessionFeatureExt for O {}
