// This file was generated by gir (https://github.com/gtk-rs/gir)
// from 
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

#include "manual.h"
#include <stdalign.h>
#include <stdio.h>

int main() {
    printf("%s;%zu;%zu\n", "SoupAuth", sizeof(SoupAuth), alignof(SoupAuth));
    printf("%s;%zu;%zu\n", "SoupAuthClass", sizeof(SoupAuthClass), alignof(SoupAuthClass));
    printf("%s;%zu;%zu\n", "SoupAuthDomain", sizeof(SoupAuthDomain), alignof(SoupAuthDomain));
    printf("%s;%zu;%zu\n", "SoupAuthDomainBasicClass", sizeof(SoupAuthDomainBasicClass), alignof(SoupAuthDomainBasicClass));
    printf("%s;%zu;%zu\n", "SoupAuthDomainClass", sizeof(SoupAuthDomainClass), alignof(SoupAuthDomainClass));
    printf("%s;%zu;%zu\n", "SoupAuthDomainDigestClass", sizeof(SoupAuthDomainDigestClass), alignof(SoupAuthDomainDigestClass));
    printf("%s;%zu;%zu\n", "SoupAuthManagerClass", sizeof(SoupAuthManagerClass), alignof(SoupAuthManagerClass));
    printf("%s;%zu;%zu\n", "SoupCache", sizeof(SoupCache), alignof(SoupCache));
    printf("%s;%zu;%zu\n", "SoupCacheClass", sizeof(SoupCacheClass), alignof(SoupCacheClass));
    printf("%s;%zu;%zu\n", "SoupCacheType", sizeof(SoupCacheType), alignof(SoupCacheType));
    printf("%s;%zu;%zu\n", "SoupCacheability", sizeof(SoupCacheability), alignof(SoupCacheability));
    printf("%s;%zu;%zu\n", "SoupContentDecoderClass", sizeof(SoupContentDecoderClass), alignof(SoupContentDecoderClass));
    printf("%s;%zu;%zu\n", "SoupContentSnifferClass", sizeof(SoupContentSnifferClass), alignof(SoupContentSnifferClass));
    printf("%s;%zu;%zu\n", "SoupCookieJar", sizeof(SoupCookieJar), alignof(SoupCookieJar));
    printf("%s;%zu;%zu\n", "SoupCookieJarAcceptPolicy", sizeof(SoupCookieJarAcceptPolicy), alignof(SoupCookieJarAcceptPolicy));
    printf("%s;%zu;%zu\n", "SoupCookieJarClass", sizeof(SoupCookieJarClass), alignof(SoupCookieJarClass));
    printf("%s;%zu;%zu\n", "SoupCookieJarDBClass", sizeof(SoupCookieJarDBClass), alignof(SoupCookieJarDBClass));
    printf("%s;%zu;%zu\n", "SoupCookieJarTextClass", sizeof(SoupCookieJarTextClass), alignof(SoupCookieJarTextClass));
    printf("%s;%zu;%zu\n", "SoupDateFormat", sizeof(SoupDateFormat), alignof(SoupDateFormat));
    printf("%s;%zu;%zu\n", "SoupEncoding", sizeof(SoupEncoding), alignof(SoupEncoding));
    printf("%s;%zu;%zu\n", "SoupExpectation", sizeof(SoupExpectation), alignof(SoupExpectation));
    printf("%s;%zu;%zu\n", "SoupHSTSEnforcer", sizeof(SoupHSTSEnforcer), alignof(SoupHSTSEnforcer));
    printf("%s;%zu;%zu\n", "SoupHSTSEnforcerClass", sizeof(SoupHSTSEnforcerClass), alignof(SoupHSTSEnforcerClass));
    printf("%s;%zu;%zu\n", "SoupHSTSEnforcerDBClass", sizeof(SoupHSTSEnforcerDBClass), alignof(SoupHSTSEnforcerDBClass));
    printf("%s;%zu;%zu\n", "SoupHTTPVersion", sizeof(SoupHTTPVersion), alignof(SoupHTTPVersion));
    printf("%s;%zu;%zu\n", "SoupLoggerClass", sizeof(SoupLoggerClass), alignof(SoupLoggerClass));
    printf("%s;%zu;%zu\n", "SoupLoggerLogLevel", sizeof(SoupLoggerLogLevel), alignof(SoupLoggerLogLevel));
    printf("%s;%zu;%zu\n", "SoupMemoryUse", sizeof(SoupMemoryUse), alignof(SoupMemoryUse));
    printf("%s;%zu;%zu\n", "SoupMessageBody", sizeof(SoupMessageBody), alignof(SoupMessageBody));
    printf("%s;%zu;%zu\n", "SoupMessageClass", sizeof(SoupMessageClass), alignof(SoupMessageClass));
    printf("%s;%zu;%zu\n", "SoupMessageFlags", sizeof(SoupMessageFlags), alignof(SoupMessageFlags));
    printf("%s;%zu;%zu\n", "SoupMessageHeadersIter", sizeof(SoupMessageHeadersIter), alignof(SoupMessageHeadersIter));
    printf("%s;%zu;%zu\n", "SoupMessageHeadersType", sizeof(SoupMessageHeadersType), alignof(SoupMessageHeadersType));
    printf("%s;%zu;%zu\n", "SoupMessagePriority", sizeof(SoupMessagePriority), alignof(SoupMessagePriority));
    printf("%s;%zu;%zu\n", "SoupMultipartInputStreamClass", sizeof(SoupMultipartInputStreamClass), alignof(SoupMultipartInputStreamClass));
    printf("%s;%zu;%zu\n", "SoupRange", sizeof(SoupRange), alignof(SoupRange));
    printf("%s;%zu;%zu\n", "SoupSameSitePolicy", sizeof(SoupSameSitePolicy), alignof(SoupSameSitePolicy));
    printf("%s;%zu;%zu\n", "SoupServer", sizeof(SoupServer), alignof(SoupServer));
    printf("%s;%zu;%zu\n", "SoupServerClass", sizeof(SoupServerClass), alignof(SoupServerClass));
    printf("%s;%zu;%zu\n", "SoupServerListenOptions", sizeof(SoupServerListenOptions), alignof(SoupServerListenOptions));
    printf("%s;%zu;%zu\n", "SoupServerMessageClass", sizeof(SoupServerMessageClass), alignof(SoupServerMessageClass));
    printf("%s;%zu;%zu\n", "SoupSession", sizeof(SoupSession), alignof(SoupSession));
    printf("%s;%zu;%zu\n", "SoupSessionClass", sizeof(SoupSessionClass), alignof(SoupSessionClass));
    printf("%s;%zu;%zu\n", "SoupSessionError", sizeof(SoupSessionError), alignof(SoupSessionError));
    printf("%s;%zu;%zu\n", "SoupStatus", sizeof(SoupStatus), alignof(SoupStatus));
    printf("%s;%zu;%zu\n", "SoupTLDError", sizeof(SoupTLDError), alignof(SoupTLDError));
    printf("%s;%zu;%zu\n", "SoupURIComponent", sizeof(SoupURIComponent), alignof(SoupURIComponent));
    printf("%s;%zu;%zu\n", "SoupWebsocketCloseCode", sizeof(SoupWebsocketCloseCode), alignof(SoupWebsocketCloseCode));
    printf("%s;%zu;%zu\n", "SoupWebsocketConnectionClass", sizeof(SoupWebsocketConnectionClass), alignof(SoupWebsocketConnectionClass));
    printf("%s;%zu;%zu\n", "SoupWebsocketConnectionType", sizeof(SoupWebsocketConnectionType), alignof(SoupWebsocketConnectionType));
    printf("%s;%zu;%zu\n", "SoupWebsocketDataType", sizeof(SoupWebsocketDataType), alignof(SoupWebsocketDataType));
    printf("%s;%zu;%zu\n", "SoupWebsocketError", sizeof(SoupWebsocketError), alignof(SoupWebsocketError));
    printf("%s;%zu;%zu\n", "SoupWebsocketExtension", sizeof(SoupWebsocketExtension), alignof(SoupWebsocketExtension));
    printf("%s;%zu;%zu\n", "SoupWebsocketExtensionClass", sizeof(SoupWebsocketExtensionClass), alignof(SoupWebsocketExtensionClass));
    printf("%s;%zu;%zu\n", "SoupWebsocketExtensionDeflateClass", sizeof(SoupWebsocketExtensionDeflateClass), alignof(SoupWebsocketExtensionDeflateClass));
    printf("%s;%zu;%zu\n", "SoupWebsocketExtensionManagerClass", sizeof(SoupWebsocketExtensionManagerClass), alignof(SoupWebsocketExtensionManagerClass));
    printf("%s;%zu;%zu\n", "SoupWebsocketState", sizeof(SoupWebsocketState), alignof(SoupWebsocketState));
    return 0;
}
