// This file was generated by gir (https://github.com/gtk-rs/gir)
// from 
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

#include "manual.h"
#include <stdio.h>

#define PRINT_CONSTANT(CONSTANT_NAME) \
    printf("%s;", #CONSTANT_NAME); \
    printf(_Generic((CONSTANT_NAME), \
                    char *: "%s", \
                    const char *: "%s", \
                    char: "%c", \
                    signed char: "%hhd", \
                    unsigned char: "%hhu", \
                    short int: "%hd", \
                    unsigned short int: "%hu", \
                    int: "%d", \
                    unsigned int: "%u", \
                    long: "%ld", \
                    unsigned long: "%lu", \
                    long long: "%lld", \
                    unsigned long long: "%llu", \
                    float: "%f", \
                    double: "%f", \
                    long double: "%ld"), \
           CONSTANT_NAME); \
    printf("\n");

int main() {
    PRINT_CONSTANT((guint) SOUP_CACHE_CACHEABLE);
    PRINT_CONSTANT((guint) SOUP_CACHE_INVALIDATES);
    PRINT_CONSTANT((gint) SOUP_CACHE_SHARED);
    PRINT_CONSTANT((gint) SOUP_CACHE_SINGLE_USER);
    PRINT_CONSTANT((guint) SOUP_CACHE_UNCACHEABLE);
    PRINT_CONSTANT((guint) SOUP_CACHE_VALIDATES);
    PRINT_CONSTANT((gint) SOUP_COOKIE_JAR_ACCEPT_ALWAYS);
    PRINT_CONSTANT((gint) SOUP_COOKIE_JAR_ACCEPT_GRANDFATHERED_THIRD_PARTY);
    PRINT_CONSTANT((gint) SOUP_COOKIE_JAR_ACCEPT_NEVER);
    PRINT_CONSTANT((gint) SOUP_COOKIE_JAR_ACCEPT_NO_THIRD_PARTY);
    PRINT_CONSTANT(SOUP_COOKIE_MAX_AGE_ONE_DAY);
    PRINT_CONSTANT(SOUP_COOKIE_MAX_AGE_ONE_HOUR);
    PRINT_CONSTANT(SOUP_COOKIE_MAX_AGE_ONE_WEEK);
    PRINT_CONSTANT(SOUP_COOKIE_MAX_AGE_ONE_YEAR);
    PRINT_CONSTANT((gint) SOUP_DATE_COOKIE);
    PRINT_CONSTANT((gint) SOUP_DATE_HTTP);
    PRINT_CONSTANT((gint) SOUP_ENCODING_BYTERANGES);
    PRINT_CONSTANT((gint) SOUP_ENCODING_CHUNKED);
    PRINT_CONSTANT((gint) SOUP_ENCODING_CONTENT_LENGTH);
    PRINT_CONSTANT((gint) SOUP_ENCODING_EOF);
    PRINT_CONSTANT((gint) SOUP_ENCODING_NONE);
    PRINT_CONSTANT((gint) SOUP_ENCODING_UNRECOGNIZED);
    PRINT_CONSTANT((guint) SOUP_EXPECTATION_CONTINUE);
    PRINT_CONSTANT((guint) SOUP_EXPECTATION_UNRECOGNIZED);
    PRINT_CONSTANT(SOUP_FORM_MIME_TYPE_MULTIPART);
    PRINT_CONSTANT(SOUP_FORM_MIME_TYPE_URLENCODED);
    PRINT_CONSTANT(SOUP_HSTS_POLICY_MAX_AGE_PAST);
    PRINT_CONSTANT((gint) SOUP_HTTP_1_0);
    PRINT_CONSTANT((gint) SOUP_HTTP_1_1);
    PRINT_CONSTANT((gint) SOUP_HTTP_2_0);
    PRINT_CONSTANT(SOUP_HTTP_URI_FLAGS);
    PRINT_CONSTANT((gint) SOUP_LOGGER_LOG_BODY);
    PRINT_CONSTANT((gint) SOUP_LOGGER_LOG_HEADERS);
    PRINT_CONSTANT((gint) SOUP_LOGGER_LOG_MINIMAL);
    PRINT_CONSTANT((gint) SOUP_LOGGER_LOG_NONE);
    PRINT_CONSTANT((gint) SOUP_MEMORY_COPY);
    PRINT_CONSTANT((gint) SOUP_MEMORY_STATIC);
    PRINT_CONSTANT((gint) SOUP_MEMORY_TAKE);
    PRINT_CONSTANT((guint) SOUP_MESSAGE_COLLECT_METRICS);
    PRINT_CONSTANT((guint) SOUP_MESSAGE_DO_NOT_USE_AUTH_CACHE);
    PRINT_CONSTANT((gint) SOUP_MESSAGE_HEADERS_MULTIPART);
    PRINT_CONSTANT((gint) SOUP_MESSAGE_HEADERS_REQUEST);
    PRINT_CONSTANT((gint) SOUP_MESSAGE_HEADERS_RESPONSE);
    PRINT_CONSTANT((guint) SOUP_MESSAGE_IDEMPOTENT);
    PRINT_CONSTANT((guint) SOUP_MESSAGE_NEW_CONNECTION);
    PRINT_CONSTANT((guint) SOUP_MESSAGE_NO_REDIRECT);
    PRINT_CONSTANT((gint) SOUP_MESSAGE_PRIORITY_HIGH);
    PRINT_CONSTANT((gint) SOUP_MESSAGE_PRIORITY_LOW);
    PRINT_CONSTANT((gint) SOUP_MESSAGE_PRIORITY_NORMAL);
    PRINT_CONSTANT((gint) SOUP_MESSAGE_PRIORITY_VERY_HIGH);
    PRINT_CONSTANT((gint) SOUP_MESSAGE_PRIORITY_VERY_LOW);
    PRINT_CONSTANT((gint) SOUP_SAME_SITE_POLICY_LAX);
    PRINT_CONSTANT((gint) SOUP_SAME_SITE_POLICY_NONE);
    PRINT_CONSTANT((gint) SOUP_SAME_SITE_POLICY_STRICT);
    PRINT_CONSTANT((guint) SOUP_SERVER_LISTEN_HTTPS);
    PRINT_CONSTANT((guint) SOUP_SERVER_LISTEN_IPV4_ONLY);
    PRINT_CONSTANT((guint) SOUP_SERVER_LISTEN_IPV6_ONLY);
    PRINT_CONSTANT((gint) SOUP_SESSION_ERROR_ENCODING);
    PRINT_CONSTANT((gint) SOUP_SESSION_ERROR_MESSAGE_ALREADY_IN_QUEUE);
    PRINT_CONSTANT((gint) SOUP_SESSION_ERROR_PARSING);
    PRINT_CONSTANT((gint) SOUP_SESSION_ERROR_REDIRECT_BAD_URI);
    PRINT_CONSTANT((gint) SOUP_SESSION_ERROR_REDIRECT_NO_LOCATION);
    PRINT_CONSTANT((gint) SOUP_SESSION_ERROR_TOO_MANY_REDIRECTS);
    PRINT_CONSTANT((gint) SOUP_SESSION_ERROR_TOO_MANY_RESTARTS);
    PRINT_CONSTANT((gint) SOUP_STATUS_ACCEPTED);
    PRINT_CONSTANT((gint) SOUP_STATUS_BAD_GATEWAY);
    PRINT_CONSTANT((gint) SOUP_STATUS_BAD_REQUEST);
    PRINT_CONSTANT((gint) SOUP_STATUS_CONFLICT);
    PRINT_CONSTANT((gint) SOUP_STATUS_CONTINUE);
    PRINT_CONSTANT((gint) SOUP_STATUS_CREATED);
    PRINT_CONSTANT((gint) SOUP_STATUS_EXPECTATION_FAILED);
    PRINT_CONSTANT((gint) SOUP_STATUS_FAILED_DEPENDENCY);
    PRINT_CONSTANT((gint) SOUP_STATUS_FORBIDDEN);
    PRINT_CONSTANT((gint) SOUP_STATUS_FOUND);
    PRINT_CONSTANT((gint) SOUP_STATUS_GATEWAY_TIMEOUT);
    PRINT_CONSTANT((gint) SOUP_STATUS_GONE);
    PRINT_CONSTANT((gint) SOUP_STATUS_HTTP_VERSION_NOT_SUPPORTED);
    PRINT_CONSTANT((gint) SOUP_STATUS_INSUFFICIENT_STORAGE);
    PRINT_CONSTANT((gint) SOUP_STATUS_INTERNAL_SERVER_ERROR);
    PRINT_CONSTANT((gint) SOUP_STATUS_INVALID_RANGE);
    PRINT_CONSTANT((gint) SOUP_STATUS_LENGTH_REQUIRED);
    PRINT_CONSTANT((gint) SOUP_STATUS_LOCKED);
    PRINT_CONSTANT((gint) SOUP_STATUS_METHOD_NOT_ALLOWED);
    PRINT_CONSTANT((gint) SOUP_STATUS_MISDIRECTED_REQUEST);
    PRINT_CONSTANT((gint) SOUP_STATUS_MOVED_PERMANENTLY);
    PRINT_CONSTANT((gint) SOUP_STATUS_MOVED_TEMPORARILY);
    PRINT_CONSTANT((gint) SOUP_STATUS_MULTIPLE_CHOICES);
    PRINT_CONSTANT((gint) SOUP_STATUS_MULTI_STATUS);
    PRINT_CONSTANT((gint) SOUP_STATUS_NONE);
    PRINT_CONSTANT((gint) SOUP_STATUS_NON_AUTHORITATIVE);
    PRINT_CONSTANT((gint) SOUP_STATUS_NOT_ACCEPTABLE);
    PRINT_CONSTANT((gint) SOUP_STATUS_NOT_APPEARING_IN_THIS_PROTOCOL);
    PRINT_CONSTANT((gint) SOUP_STATUS_NOT_EXTENDED);
    PRINT_CONSTANT((gint) SOUP_STATUS_NOT_FOUND);
    PRINT_CONSTANT((gint) SOUP_STATUS_NOT_IMPLEMENTED);
    PRINT_CONSTANT((gint) SOUP_STATUS_NOT_MODIFIED);
    PRINT_CONSTANT((gint) SOUP_STATUS_NO_CONTENT);
    PRINT_CONSTANT((gint) SOUP_STATUS_OK);
    PRINT_CONSTANT((gint) SOUP_STATUS_PARTIAL_CONTENT);
    PRINT_CONSTANT((gint) SOUP_STATUS_PAYMENT_REQUIRED);
    PRINT_CONSTANT((gint) SOUP_STATUS_PERMANENT_REDIRECT);
    PRINT_CONSTANT((gint) SOUP_STATUS_PRECONDITION_FAILED);
    PRINT_CONSTANT((gint) SOUP_STATUS_PROCESSING);
    PRINT_CONSTANT((gint) SOUP_STATUS_PROXY_AUTHENTICATION_REQUIRED);
    PRINT_CONSTANT((gint) SOUP_STATUS_PROXY_UNAUTHORIZED);
    PRINT_CONSTANT((gint) SOUP_STATUS_REQUESTED_RANGE_NOT_SATISFIABLE);
    PRINT_CONSTANT((gint) SOUP_STATUS_REQUEST_ENTITY_TOO_LARGE);
    PRINT_CONSTANT((gint) SOUP_STATUS_REQUEST_TIMEOUT);
    PRINT_CONSTANT((gint) SOUP_STATUS_REQUEST_URI_TOO_LONG);
    PRINT_CONSTANT((gint) SOUP_STATUS_RESET_CONTENT);
    PRINT_CONSTANT((gint) SOUP_STATUS_SEE_OTHER);
    PRINT_CONSTANT((gint) SOUP_STATUS_SERVICE_UNAVAILABLE);
    PRINT_CONSTANT((gint) SOUP_STATUS_SWITCHING_PROTOCOLS);
    PRINT_CONSTANT((gint) SOUP_STATUS_TEMPORARY_REDIRECT);
    PRINT_CONSTANT((gint) SOUP_STATUS_UNAUTHORIZED);
    PRINT_CONSTANT((gint) SOUP_STATUS_UNPROCESSABLE_ENTITY);
    PRINT_CONSTANT((gint) SOUP_STATUS_UNSUPPORTED_MEDIA_TYPE);
    PRINT_CONSTANT((gint) SOUP_STATUS_USE_PROXY);
    PRINT_CONSTANT((gint) SOUP_TLD_ERROR_INVALID_HOSTNAME);
    PRINT_CONSTANT((gint) SOUP_TLD_ERROR_IS_IP_ADDRESS);
    PRINT_CONSTANT((gint) SOUP_TLD_ERROR_NOT_ENOUGH_DOMAINS);
    PRINT_CONSTANT((gint) SOUP_TLD_ERROR_NO_BASE_DOMAIN);
    PRINT_CONSTANT((gint) SOUP_TLD_ERROR_NO_PSL_DATA);
    PRINT_CONSTANT((gint) SOUP_URI_AUTH_PARAMS);
    PRINT_CONSTANT((gint) SOUP_URI_FRAGMENT);
    PRINT_CONSTANT((gint) SOUP_URI_HOST);
    PRINT_CONSTANT((gint) SOUP_URI_NONE);
    PRINT_CONSTANT((gint) SOUP_URI_PASSWORD);
    PRINT_CONSTANT((gint) SOUP_URI_PATH);
    PRINT_CONSTANT((gint) SOUP_URI_PORT);
    PRINT_CONSTANT((gint) SOUP_URI_QUERY);
    PRINT_CONSTANT((gint) SOUP_URI_SCHEME);
    PRINT_CONSTANT((gint) SOUP_URI_USER);
    PRINT_CONSTANT((gint) SOUP_WEBSOCKET_CLOSE_ABNORMAL);
    PRINT_CONSTANT((gint) SOUP_WEBSOCKET_CLOSE_BAD_DATA);
    PRINT_CONSTANT((gint) SOUP_WEBSOCKET_CLOSE_GOING_AWAY);
    PRINT_CONSTANT((gint) SOUP_WEBSOCKET_CLOSE_NORMAL);
    PRINT_CONSTANT((gint) SOUP_WEBSOCKET_CLOSE_NO_EXTENSION);
    PRINT_CONSTANT((gint) SOUP_WEBSOCKET_CLOSE_NO_STATUS);
    PRINT_CONSTANT((gint) SOUP_WEBSOCKET_CLOSE_POLICY_VIOLATION);
    PRINT_CONSTANT((gint) SOUP_WEBSOCKET_CLOSE_PROTOCOL_ERROR);
    PRINT_CONSTANT((gint) SOUP_WEBSOCKET_CLOSE_SERVER_ERROR);
    PRINT_CONSTANT((gint) SOUP_WEBSOCKET_CLOSE_TLS_HANDSHAKE);
    PRINT_CONSTANT((gint) SOUP_WEBSOCKET_CLOSE_TOO_BIG);
    PRINT_CONSTANT((gint) SOUP_WEBSOCKET_CLOSE_UNSUPPORTED_DATA);
    PRINT_CONSTANT((gint) SOUP_WEBSOCKET_CONNECTION_CLIENT);
    PRINT_CONSTANT((gint) SOUP_WEBSOCKET_CONNECTION_SERVER);
    PRINT_CONSTANT((gint) SOUP_WEBSOCKET_CONNECTION_UNKNOWN);
    PRINT_CONSTANT((gint) SOUP_WEBSOCKET_DATA_BINARY);
    PRINT_CONSTANT((gint) SOUP_WEBSOCKET_DATA_TEXT);
    PRINT_CONSTANT((gint) SOUP_WEBSOCKET_ERROR_BAD_HANDSHAKE);
    PRINT_CONSTANT((gint) SOUP_WEBSOCKET_ERROR_BAD_ORIGIN);
    PRINT_CONSTANT((gint) SOUP_WEBSOCKET_ERROR_FAILED);
    PRINT_CONSTANT((gint) SOUP_WEBSOCKET_ERROR_NOT_WEBSOCKET);
    PRINT_CONSTANT((gint) SOUP_WEBSOCKET_STATE_CLOSED);
    PRINT_CONSTANT((gint) SOUP_WEBSOCKET_STATE_CLOSING);
    PRINT_CONSTANT((gint) SOUP_WEBSOCKET_STATE_OPEN);
    return 0;
}
