// This file was generated by gir (https://github.com/gtk-rs/gir)
// from
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use crate::{ffi, CookieJar, CookieJarAcceptPolicy, SessionFeature};
use glib::{prelude::*, translate::*};

glib::wrapper! {
    #[doc(alias = "SoupCookieJarDB")]
    pub struct CookieJarDB(Object<ffi::SoupCookieJarDB, ffi::SoupCookieJarDBClass>) @extends CookieJar, @implements SessionFeature;

    match fn {
        type_ => || ffi::soup_cookie_jar_db_get_type(),
    }
}

impl CookieJarDB {
    #[doc(alias = "soup_cookie_jar_db_new")]
    pub fn new(filename: &str, read_only: bool) -> CookieJarDB {
        assert_initialized_main_thread!();
        unsafe {
            CookieJar::from_glib_full(ffi::soup_cookie_jar_db_new(
                filename.to_glib_none().0,
                read_only.into_glib(),
            ))
            .unsafe_cast()
        }
    }

    // rustdoc-stripper-ignore-next
    /// Creates a new builder-pattern struct instance to construct [`CookieJarDB`] objects.
    ///
    /// This method returns an instance of [`CookieJarDBBuilder`](crate::builders::CookieJarDBBuilder) which can be used to create [`CookieJarDB`] objects.
    pub fn builder() -> CookieJarDBBuilder {
        CookieJarDBBuilder::new()
    }

    pub fn filename(&self) -> Option<glib::GString> {
        ObjectExt::property(self, "filename")
    }
}

impl Default for CookieJarDB {
    fn default() -> Self {
        glib::object::Object::new::<Self>()
    }
}

// rustdoc-stripper-ignore-next
/// A [builder-pattern] type to construct [`CookieJarDB`] objects.
///
/// [builder-pattern]: https://doc.rust-lang.org/1.0.0/style/ownership/builders.html
#[must_use = "The builder must be built to be used"]
pub struct CookieJarDBBuilder {
    builder: glib::object::ObjectBuilder<'static, CookieJarDB>,
}

impl CookieJarDBBuilder {
    fn new() -> Self {
        Self {
            builder: glib::object::Object::builder(),
        }
    }

    pub fn filename(self, filename: impl Into<glib::GString>) -> Self {
        Self {
            builder: self.builder.property("filename", filename.into()),
        }
    }

    pub fn accept_policy(self, accept_policy: CookieJarAcceptPolicy) -> Self {
        Self {
            builder: self.builder.property("accept-policy", accept_policy),
        }
    }

    pub fn read_only(self, read_only: bool) -> Self {
        Self {
            builder: self.builder.property("read-only", read_only),
        }
    }

    // rustdoc-stripper-ignore-next
    /// Build the [`CookieJarDB`].
    #[must_use = "Building the object from the builder is usually expensive and is not expected to have side effects"]
    pub fn build(self) -> CookieJarDB {
        assert_initialized_main_thread!();
        self.builder.build()
    }
}
