// This file was generated by gir (https://github.com/gtk-rs/gir)
// from
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use crate::{ffi, Auth};
use glib::prelude::*;

glib::wrapper! {
    #[doc(alias = "SoupAuthDigest")]
    pub struct AuthDigest(Object<ffi::SoupAuthDigest>) @extends Auth;

    match fn {
        type_ => || ffi::soup_auth_digest_get_type(),
    }
}

impl AuthDigest {
    // rustdoc-stripper-ignore-next
    /// Creates a new builder-pattern struct instance to construct [`AuthDigest`] objects.
    ///
    /// This method returns an instance of [`AuthDigestBuilder`](crate::builders::AuthDigestBuilder) which can be used to create [`AuthDigest`] objects.
    pub fn builder() -> AuthDigestBuilder {
        AuthDigestBuilder::new()
    }
}

// rustdoc-stripper-ignore-next
/// A [builder-pattern] type to construct [`AuthDigest`] objects.
///
/// [builder-pattern]: https://doc.rust-lang.org/1.0.0/style/ownership/builders.html
#[must_use = "The builder must be built to be used"]
pub struct AuthDigestBuilder {
    builder: glib::object::ObjectBuilder<'static, AuthDigest>,
}

impl AuthDigestBuilder {
    fn new() -> Self {
        Self {
            builder: glib::object::Object::builder(),
        }
    }

    pub fn authority(self, authority: impl Into<glib::GString>) -> Self {
        Self {
            builder: self.builder.property("authority", authority.into()),
        }
    }

    pub fn is_for_proxy(self, is_for_proxy: bool) -> Self {
        Self {
            builder: self.builder.property("is-for-proxy", is_for_proxy),
        }
    }

    pub fn realm(self, realm: impl Into<glib::GString>) -> Self {
        Self {
            builder: self.builder.property("realm", realm.into()),
        }
    }

    // rustdoc-stripper-ignore-next
    /// Build the [`AuthDigest`].
    #[must_use = "Building the object from the builder is usually expensive and is not expected to have side effects"]
    pub fn build(self) -> AuthDigest {
        assert_initialized_main_thread!();
        self.builder.build()
    }
}
